﻿-- ====================================================================
--  UIManager.lua
--  Contains functions used for operations that impact what the user
--  sees.
-- ====================================================================

-- Windows
local                 mainWindow = TortaParentFrame;
local                  startPage = TortaParentFrameStartPage;
local                   mainPage = TortaParentFrameMainPage;
local             explorerWindow = TortaParentFrameMainPageExplorerSubWindow;
local            clDisplayWindow = TortaParentFrameMainPageCLDisplaySubWindow;
local            nbDisplayWindow = TortaParentFrameMainPageNBDisplaySubWindow;

-- Misc. Widgets
local          explorerScrollBar = TortaParentFrameMainPageExplorerSubWindowScrollBar;
local                clScrollBar = TortaParentFrameMainPageCLDisplaySubWindowScrollBar;
local           noLoadedListText = TortaParentFrameMainPageCLDisplaySubWindowNoLoadedListText;
local                noTasksText = TortaParentFrameMainPageCLDisplaySubWindowNoTasksText;
local                clTopButton = TortaParentFrameMainPageCLDisplaySubWindowTaskButton1;
local              cl3HeaderText = TortaParentFrameMainPageCLDisplaySubWindowHeaderText;
local                 taskEditor = TortaParentFrameMainPageTaskEditorSubWindow;
local                  clTaskBar = TortaParentFrameMainPageCLTaskbar;
local                  nbTaskBar = TortaParentFrameMainPageNBTaskbar;
local                 editTaskEB = TortaParentFrameMainPageTaskEditorSubWindowScrollFrameEditBox;
local           uncheckAllButton = TortaParentFrameMainPageCLTaskbarUncheckAllButton;
local             checkAllButton = TortaParentFrameMainPageCLTaskbarCheckAllButton;
local              addTaskButton = TortaParentFrameMainPageCLTaskbarAddTaskButton;
local           removeTaskButton = TortaParentFrameMainPageCLTaskbarRemoveTaskButton;
local        explorerNoListsText = TortaParentFrameMainPageExplorerSubWindowNoListsText;
local        explorerNoNtbksText = TortaParentFrameMainPageExplorerSubWindowNoNotebooksText;
local                statsButton = TortaParentFrameMainPageCLTaskbarStatsButton;
local                welcomeText = TortaParentFrameStartPageWelcomeSubWindowScrollFrameContentsText;

local EXPLORER_LISTBUTTON_HEIGHT = 16;
local Torta = Torta;

noLoadedListText:SetText("|cffffd000What? No loaded list? I... I feel ignored. /cry|r\nYou currently do not have a list loaded. You can either select a list in Turtle Explorer or create a new one by right-clicking inside Turtle Explorer.");
noTasksText:SetText("|cffffd000Woah, no tasks!|r\nThe currently loaded list does not appear to have any tasks in it. You can create new tasks for this list by clicking the \"Add A Task\" button below.");
welcomeText:SetText([[
Thank you for downloading Torta! If you have any problems while using this addon and the solution is not in this page, you can find additional support at either of the following pages:

http://www.wowinterface.com/downloads/info11619-Torta.html
http://www.wowace.com/projects/torta/

Torta is BSD-licensed, which makes it |cffffd000free and open-source software|r. It is free in the sense of both "free beer" and "free speech". The full license text can be found in LICENSE.txt in this addon's folder.

If you experience trouble with a modified version of Torta, try a version from an official source. See the URLs above.

|cffffd000The Torta User's Manual|r

|cffffd000Table of Contents|r
I. What's new in Torta?
II. What's in a name?
III. Torta Checklist


|cffffd000Section I. What's new in Torta?|r
Torta is the next generation of the Checklist addon, keeping most of the latter's features while providing a solid platform for additional ones.

Under the hood, Torta features a re-designed database system which takes a smaller memory footprint and is easier to work with for the developer.

(Add more stuff here)


|cffffd000Section II. What's in a name?|r
Torta was picked to be the name for this addon because:

Torta is the name of a tortoise in Tanaris. Tortoises are tough. I want my addons to be tough and not prone to bugs.

In the Philippines, 'torta' refers to a kind of omelette made with eggs, ground meat and sometimes minced onion and potato. (See http://en.wikipedia.org/wiki/Torta)

The SVN client that I use is TortoiseSVN. More turtle insanity, and...

The name "Checklist" no longer fit the addon, as it left out the notes module added in the latter part of Checklist2's lifespan. "Torta" does not describe any one part of the addon, leaving it to be free to expand, contract, bounce, or cause some other equal and opposite reaction as described in Newton's third law of motion.


|cffffd000Section III. Torta Checklist|r

|cffffd000Section III. Torta Notebook|r

|cffffd000Section n. Quirks|r

"Why does this help text vanish when the window is scaling in/scaling out?"
When this window scales in/out with this text visible, WoW constantly recalculates stuff concerning the size of this text, the size of the scroll frame it's in, and maybe other stuff. This causes quite a performance hit, enough to make me hide the text before any re-scaling is done.

"Why is the shortcut to saving task text shift-ESC? You should just use the normal way of doing things, i.e. CTRL-Enter!"
When set to be multi-line, WoW's editbox widget does not tell the addon when enter has been pressed. I used a multi-line editbox so more text can be visible at once, unlike in earlier versions of Checklist.

]]);
welcomeText:SetAlpha(0);
welcomeText:Hide();
welcomeText.finishedFunc = function()
  if welcomeText:GetAlpha() == 0 then
    print("Text hidden!");
    welcomeText:Hide();
  end
end

-- Explorer subwindow stuff
Torta.gui.exRecycleBin = {};
Torta.gui.exListButtons = { TortaParentFrameMainPageExplorerSubWindowListButton1 };
-- Checklist subwindow stuff
Torta.gui.clRecycleBin = {};
Torta.gui.clListButtons = { TortaParentFrameMainPageCLDisplaySubWindowTaskButton1 };

-- ====================================================================
--  Allow public access to our "shortcuts"
-- ====================================================================
Torta.gui.elements = {};
local e = Torta.gui.elements;
e.mainWindow          = mainWindow;
e.startPage           = startPage;
e.explorerWindow      = explorerWindow;
e.explorerScrollBar   = explorerScrollBar;
e.clDisplayWindow     = clDisplayWindow;
e.clScrollBar         = clScrollBar;
e.noLoadedListText    = noLoadedListText;
e.clTopButton         = clTopButton;
e.cl3HeaderText       = cl3HeaderText;
e.taskEditor          = taskEditor;
e.clTaskBar           = clTaskBar;
e.nbTaskBar           = nbTaskBar;
e.editTaskEB          = editTaskEB;
e.uncheckAllButton    = uncheckAllButton;
e.checkAllButton      = checkAllButton;
e.addTaskButton       = addTaskButton;
e.removeTaskButton    = removeTaskButton;
e.explorerNoListsText = explorerNoListsText;
e.explorerNoNtbksText = explorerNoNtbksText
e.statsButton         = statsButton;
e = nil;
-- ====================================================================

function Torta.gui.scalingFinishedFunc(frame)
  frame:Hide();
end

local FADETIME = 0.25;
function Torta.gui:ToggleWindow(window)
  window = window or mainWindow;
  if self:IsFrameScaling(window) or UIFrameIsFading(window) then
    return;
  end
  if not window:IsVisible() then
    window.scalingFinishedFunc = nil;
    window:Show();
    self:ScaleIn(window, FADETIME, 0.8, 1);
    UIFrameFadeIn(window, FADETIME, 0, 1);
  else
    if window == mainWindow then
      welcomeText:Hide();
    end
    window.scalingFinishedFunc = self.scalingFinishedFunc;
    self:ScaleOut(window, FADETIME, 1, 0.7);
    UIFrameFadeOut(window, FADETIME, 1, 0);
  end
end


function Torta.gui:OnTabClicked(tabID)
  assert(type(tabID) == "number");
  PanelTemplates_SetTab(mainWindow, tabID);
  if Torta.cl3.HideTaskEditor then
    Torta.cl3:HideTaskEditor();
  end
  if tabID == 1 then
    startPage:Show();
    mainPage:Hide();
  else
    self.tab = tabID;
    startPage:Hide();
    mainPage:Show();
    self:exUpdateScrollFrame();
    if tabID == 2 then
      clDisplayWindow:Show();
      clTaskBar:Show();
      
      nbDisplayWindow:Hide();
      nbTaskBar:Hide();
    else
      clDisplayWindow:Hide();
      clTaskBar:Hide();
      
      nbDisplayWindow:Show();
      nbTaskBar:Show();
    end
  end
end


function Torta.gui:Initialize()
  
end

-- Explorer listbox code
function Torta.gui:CheckExplorerWindow()
  local numButtonsCanFit = floor((explorerWindow:GetHeight() - (20 + 2 + 2)) / EXPLORER_LISTBUTTON_HEIGHT);
  local numButtons = #self.exListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    self:CreateExplorerButtons(numButtonsCanFit - numButtons);
  else
    self:RemoveExplorerButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons + 1;
    local button = (self.exRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        explorerWindow,
        "TortaExplorerButtonTemplate"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.exListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.exListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.exListButtons[id] = button;
  end
  self:exUpdateScrollFrame();
end

function Torta.gui:RemoveExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons
    self.exRecycleBin[id] = self.exListButtons[id];
    self.exListButtons[id]:Hide();
    self.exListButtons[id] = nil;
  end
  self:exUpdateScrollFrame();
end


function Torta.gui:exUpdateScrollFrame()
  self = Torta.gui;
  local lineplusoffset;
  local button;
  explorerNoListsText:Hide();
  explorerNoNtbksText:Hide();
  
  if self.tab == 2 then -- if Torta Checklist is selected
    if #Torta.db.profile.lists > 0 then
      local lists = Torta.db.profile.lists;
      FauxScrollFrame_Update(explorerScrollBar, #Torta.db.profile.lists, #self.exListButtons, 16, nil, nil, nil, nil, nil, nil, true)
      local loadedListID = Torta.db.profile.cfg.loadedList;
      for line = 1, #self.exListButtons do
        lineplusoffset = line + FauxScrollFrame_GetOffset(explorerScrollBar);
        button = self.exListButtons[line];
        if lineplusoffset <= #Torta.db.profile.lists then
          if not lists[lineplusoffset] then return end
          button.text:SetText(lists[lineplusoffset].meta.listName);
          button:SetID(lineplusoffset);
          
          if lineplusoffset ~= loadedListID then
            button:UnlockHighlight();
          else
            button:LockHighlight();
          end
          
          button.topDropHandler.destLoc = lineplusoffset;
          button.bottomDropHandler.destLoc = lineplusoffset + 1;
          
          button:Show();
        else
          button:Hide();
        end
      end
    else
      explorerScrollBar:Hide();
      explorerNoListsText:Show();
      for i = 1, #self.exListButtons do
        self.exListButtons[i]:Hide();
      end
    end
  else  -- if Torta Notebook is selected
    if #Torta.db.profile.notes > 0 then
      local notes = Torta.db.profile.notes;
      FauxScrollFrame_Update(explorerScrollBar, #Torta.db.profile.notes, #self.exListButtons, 16, nil, nil, nil, nil, nil, nil, true)
      local loadedListID = Torta.db.profile.cfg.loadedList;
      for line = 1, #self.exListButtons do
        lineplusoffset = line + FauxScrollFrame_GetOffset(explorerScrollBar);
        button = self.exListButtons[line];
        if lineplusoffset <= #Torta.db.profile.notes then
          if not notes[lineplusoffset] then return end
          button:SetText(notes[lineplusoffset].meta.name);
          button:SetID(lineplusoffset);
          
          if lineplusoffset ~= loadedListID then
            button:UnlockHighlight();
          else
            button:LockHighlight();
          end
          
          button:Show();
        else
          button:Hide();
        end
      end
    else
      explorerScrollBar:Hide();
      explorerNoNtbksText:Show();
      for i = 1, #self.exListButtons do
        self.exListButtons[i]:Hide();
      end
    end
  end
end



function Torta.gui:clCheckButtons()
  local numButtonsCanFit = floor((clDisplayWindow:GetHeight() - (20 + 2 + 2)) / 32);
  local numButtons = #self.clListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    self:CreateCLButtons(numButtonsCanFit - numButtons);
  else
    self:RemoveCLButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons + 1;
    local button = (self.clRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        clDisplayWindow,
        "TortaChecklistTaskButton"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.clListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.clListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.clListButtons[id] = button;
  end
  self:clUpdateScrollFrame();
end

function Torta.gui:RemoveCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons
    self.clRecycleBin[id] = self.clListButtons[id];
    self.clListButtons[id]:Hide();
    self.clListButtons[id] = nil;
  end
  self:clUpdateScrollFrame();
end

-- CHECKLIST DISPLAY FUNCTION
function Torta.gui:clUpdateScrollFrame(isFromOnShow)
  self = Torta.gui;
  local lineplusoffset;
  local button;
  
  noTasksText:Hide();
  noLoadedListText:Hide();
  
  clDisplayWindow:Show();
  nbDisplayWindow:Hide();
  if self.listData then
    noLoadedListText:Hide();
    cl3HeaderText:SetText(self.listData.meta.listName);
    FauxScrollFrame_Update(clScrollBar, #self.listData.text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
    if not isFromOnShow then
      clScrollBar:Show();
    end
    for line = 1, #self.clListButtons do
      lineplusoffset = line + FauxScrollFrame_GetOffset(clScrollBar);
      button = self.clListButtons[line];
      if lineplusoffset <= #self.listData.text then
        button.text:SetFormattedText("|cffffd000%i.)|r %s", lineplusoffset, self.listData.text[lineplusoffset]);
        button.check:SetChecked(self.listData.stat[lineplusoffset]);
        button:SetID(lineplusoffset);
        button.check:SetID(lineplusoffset);
        button:SetID(lineplusoffset);
        
        local topText = "Insert above task " .. lineplusoffset;
        local btmText = "Insert below task " .. lineplusoffset;
        
        button.topDropHandler:SetFormattedText("%s - %s - %s", topText, topText, topText);
        button.topDropHandler.destLoc = lineplusoffset;
        button.bottomDropHandler:SetFormattedText("%s - %s - %s", btmText, btmText, btmText);
        button.bottomDropHandler.destLoc = lineplusoffset + 1;
        
        if lineplusoffset ~= Torta.cl3.openTask then
          button:UnlockHighlight();
        else
          button:LockHighlight();
        end
        
        button:Show();
      else
        button:Hide();
      end
    end
    if #self.listData.text == 0 then
      noTasksText:Show();
      clScrollBar:Hide()
    else
      noTasksText:Hide();
      clScrollBar:Show();
    end
  else
    noLoadedListText:Show();
    clScrollBar:Hide();
    for line = 1, #self.clListButtons do
      self.clListButtons[line]:Hide();
    end
  end
  Torta.cl3:UpdateStats();
end


function Torta.gui:IsTaskVisible(id)
  for k,button in pairs(self.clListButtons) do
    if button:GetID() == id then
      return true;
    end
  end
  return false;
end


function Torta.gui:MakeTaskVisible(taskID)
  assert(type(taskID) == "number", "Assertion failed: method MakeTaskVisible requires a number parameter (taskID)");
  local scrollBar = getglobal(clScrollBar:GetName() .. "ScrollBar");
  local offset = FauxScrollFrame_GetOffset(clScrollBar);
  for i = 1, 1000 do  -- just so we don't go into an infinite loop
    if self:IsTaskVisible(taskID) then
      break;
    end
    
    FauxScrollFrame_SetOffset(clScrollBar, offset);
    FauxScrollFrame_Update(clScrollBar, #Torta.cl3:GetLoadedList().text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
    self:clCheckButtons();
    self:clUpdateScrollFrame();
    offset = offset + 1
  end
  scrollBar:SetValue(floor(offset * 32 + 0.5));
  -- print(offset, scrollBar:GetValue());
end

-- ====================================================================
--  Drag and drop functions
-- ====================================================================
function Torta.gui:clShowDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
    UIFrameFadeOut(button.check, 0.3, button.check:GetAlpha(), 0.5);
  end
end

function Torta.gui:clHideDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
    UIFrameFadeIn(button.check, 0.3, button.check:GetAlpha(), 1);
  end
end

function Torta.gui:clTaskDragStart(taskButton)
  PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
  local text = taskButton.text:GetText();
  local stat = taskButton.check:GetChecked();
  TortaDragThingy.text:SetText(text);
  TortaDragThingy.check:SetChecked(stat);
  TortaDragThingy:Show();
  UIFrameFadeIn(TortaDragThingy, 0.25, 0, 1);
  self:clShowDropHandlers();
  self.dragSrcID = taskButton:GetID();
end

function Torta.gui:clTaskDragStop()
  PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
  TortaDragThingy:Hide();
  if GetMouseFocus().tortaDropType == "CLTASK" then
    Torta.cl3:MoveTask(self.dragSrcID, GetMouseFocus().destLoc);
    GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
  end
  self:clHideDropHandlers();
end


function Torta.gui:clexShowDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
  end
end

function Torta.gui:clexHideDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
  end
end

function Torta.gui:clListDragStart(listButton)
  PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
  local text = listButton.text:GetText();
  TortaCLListDragThingy.text:SetText(text);
  TortaCLListDragThingy:Show();
  UIFrameFadeIn(TortaCLListDragThingy, 0.25, 0, 1);
  self:clexShowDropHandlers();
  self.dragSrcID = listButton:GetID();
end

function Torta.gui:clListDragStop()
  PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
  TortaCLListDragThingy:Hide();
  if GetMouseFocus().tortaDropType == "CLLIST" then
    Torta.cl3:MoveList(self.dragSrcID, GetMouseFocus().destLoc);
    GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
  end
  self:clexHideDropHandlers();
end

-- Note:
-- Add all UI management functions ABOVE!

-- ====================================================================
-- Dropdown menu stuff
-- ====================================================================
local clExplorerDropdownInfo = {
  {
    text = "Tortoise Explorer",
    isTitle = true
  },
  {
    text = "Create a new list",
    func = function() Torta.cl3:CreateList(); end
  },
  {
    text = "Profiles...",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}
local clListDropdownInfo = {
  {
    text = "<will get replaced by list name>",
    isTitle = true
  },
  {
    text = "Rename this list",
    func = function() Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "Delete this list",
    func = function() Torta.cl3:DeleteList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Create a new list",
    func = function() Torta.cl3:CreateList(); end,
  },
  {
    text = "Profiles...",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}
local dropdown = CreateFrame("Frame", "TortaDropDownMenuFrame", explorerWindow, "UIDropDownMenuTemplate");

function Torta.gui:clShowButtonDropdown(button)
  assert(button);
  self.selectedExplorerButton = button;
  clListDropdownInfo[1].text = button.text:GetText();
  EasyMenu(clListDropdownInfo, dropdown, "cursor", 0, 0, "MENU", 3);
end


function Torta.gui:clShowExplorerDropdownMenu()
  EasyMenu(clExplorerDropdownInfo, dropdown, "cursor", 0, 0, "MENU", 3);
end

-- ====================================================================
--  Dialogs
-- ====================================================================
StaticPopupDialogs["TORTA_CLRENAMELIST"] = {
  text = "Please enter the new name for the list currently named \"|cffffd000%s|r\":",
  OnAccept = function(frame)
    local newName = getglobal(frame:GetName() .. "EditBox"):GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    frame:Hide();
  end,
  EditBoxOnEnterPressed = function(editBox)
    local newName = editBox:GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    editBox:GetParent():Hide();
  end,
  EditBoxOnEscapePressed = function(self)
    self:GetParent():Hide();
  end,
  button1 = ACCEPT,
  button2 = CANCEL,
  whileDead = 1,
  timeout = 30,
  hasEditBox = 1,
  hideOnEscape = 1,
}
StaticPopupDialogs["TORTA_CLDELETELIST"] = {
  text = "Are you ABSOLUTELY sure you want to delete the list \"|cffffd000%s|r\"?",
  button1 = YES,
  button2 = NO,
  showAlert = true,
  OnAccept = function(self, data)
    Torta.cl3:DeleteList(data, true);
  end,
  whileDead = 1,
  timeout = 30,
  hideOnEscape = 1
}
--  Dialogs end here
-- ====================================================================

--

-- ====================================================================
--  Misc. UI init code
-- ====================================================================
FauxScrollFrame_SetOffset(explorerScrollBar, 0);
FauxScrollFrame_SetOffset(clScrollBar, 0);

mainWindow:SetMinResize(820, 450);
PanelTemplates_SetNumTabs(mainWindow, 3);
Torta.gui:OnTabClicked(1);

Torta:RegisterRevision("$Revision: 7 $");